#!/usr/bin/perl -w
#************************************************************ 
#
# Component       CNF  
#
# Synopsis        Main module
#
#
# Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
#
#************************************************************
#
# #ident "@(#)nhadm.pl 1.11 04/03/12 SMI"
#
#************************************************************

package MAIN ;

use strict;
no strict 'refs';
use VAR;
use LOG ;
use CONF;
use OS ;
use OBP ;
use RBS ;
use CGTP ;
use CMM ;
use FS ;
use CRFS ;
use SYNC ;
use WDT;
use NSM;
use COPY;

#
#  retrieving of arguments
#
my @G_step_array ;        # list of steps to be tested
my $G_step_count = 0 ;    # # of steps
my @G_master_array ;      # list of master nodes
my $G_master_count = 0 ;  # # of master nodes
my @G_peer_array ;        # list of peer nodes
my $G_peer_count = 0 ;    # # of peer nodes

#
#  list of modules with a "display" function
#
my @G_display_all = ("os" , "conf", "cmm", "cgtp" , "fs", "crfs" ) ;

#
#  stage decription for checking
#
my @G_stages = ("installation", "configuration", "starting") ;

my %G_check = (
	    "installation"  =>  [ "os" ],
	    "configuration" =>  [ "obp", "rbs", "cmm", "wdt", "nsm", 
				  "crfs", "cgtp" ],
	    "starting"      =>  [ "cmm", "crfs", "cgtp"]
	    ) ;

#
# For command
#

my $G_readcmd       = 0 ;    # reference to the element of $G_command under processing
my @G_postcommand   = () ;   # post-command under processing (array of read arguments)

#
# PRE AND POST-COMMAND PARAMETER DEFINITION
#
# nahdm -v -e <value> command -s -b <value>
#       -------------         -------------
#          pre                 post
#
# 0: short format (without -)
# 1: long format  (without --)
# 2: help string
# 3: action                   4: special depends on the action                  5: special2
#
#    READ                        both: variable to be set                          variable with default value
#    (ex: -c <filename>)                                                           (used when display help)
#
#    GET                         post: same as read but argument and 
#                                      value are stored in the post-command
#                                      array (usefull for shell script with
#                                      control of authorized parameters)
#
#    CALL                        both: Perl function to be called as soon
#    (ex: -v calls                     as argument is read
#         LOG::incr_verbose)
#
#    COPY                        post: store the argument in the post-command
#    (ex: -<flags>)                          array if it appears
#

my $G_prmshort   = 0 ;
my $G_prmlong    = 1 ;
my $G_prmhlp     = 2 ; 
my $G_prmaction  = 3 ;
my $G_prmspecial = 4 ;
my $G_prmspecial2= 5 ;

my @G_global_parameters = 
  (
   [ "-h", "--help",
     convMsg("display this help"), 
     "CALL", "help" ] ,
   [ "-v", "--verbose", 
     convMsg("debug information (each occurence increases the level))"),
     "CALL", "LOG::incr_verbose" ] ,
   [ "-s", "--silent",  
     convMsg("only error messages are displayed"),
     "CALL", "LOG::set_silent" ] ,
  ) ;

my @G_check_parameters = 
  (
   [ "-f", "--fsconf",
     convMsg("specify the Foundation Services configuration file"),
     "READ", "E_fs_configuration_file", "E_default_fs_configuration_file" ] ,
  ) ;

my @G_sync_parameters = 
  (
   [ "-y", "--syncfile", 
     convMsg("specify the synchronization configuration file") ,
     "READ", "E_syncfile", "E_default_syncfile" ] ,
  ) ;

my @G_copy_parameters = 
  (
   [ "-d", "--data", 
     convMsg("specify the data file listing files to be copied") ,
     "READ", "E_datacopy_file", "" ] ,
  ) ;

#
#  array of all possibles parameters
#  this table is used for parsing, its interest is also to display parameter help
#  per command category instead of per command
#
#  1: help
#  2: reference to a specific parameter array

my $G_allhlp = 0 ;
my $G_alllst = 1 ;
my @G_all_parameters =
  ( 
   [ convMsg("general"),
     \@G_global_parameters] ,
   [ convMsg("check and display"),
     \@G_check_parameters] , 
   [ convMsg("synccheck and syncgen"),
     \@G_sync_parameters] ,
   [ convMsg("copy"),
     \@G_copy_parameters] ,
  ) ;

#
#  COMMAND DEFINITION
#
#  nhadm [arguments] check [postcommand]
#
#  1: name of the command
#  2: help string
#  3: post-command processing              4: depends on the post-command processing
#  
#     NONE                                 -
#     VALUE                                meaning of the value for help
#     VALUE_LIST                           reference of a array containing authorized values
#     PARAMETER                            reference of a parameter array definition
#               
#  5: action performed when analyze        6: depends on the action
#     is complete                          
#     
#     CALL                                 Perf function to be called
#                                          all post-command values will be passed as separate
#                                          arguments (post-command array passed by value)
#
#     SHELL                                Shell command to execute
#                                          all post-command values will be passed as arguments
#                                          (separated by space).
#
#     CONCATENATE                          the rest of the argument are stored "as is"
#                                          (separate by blanks)
#                                          useful when arguments have to be passed to a 
#                                          shell command for example or to pass a string to
#                                          a function

my $G_cmdname    = 0 ;
my $G_cmdhlp     = 1 ;
my $G_cmdpostcmd = 2 ;
my $G_cmdpostpar = 3 ;
my $G_cmdaction  = 4 ;
my $G_cmdspecial = 5 ;
my @G_command =
  (
   [ "check", 
     convMsg("check all (default) or part of the node"),
     "VALUE_LIST", \@G_stages,
     "CALL", "MAIN::do_check" ] ,

   [ "display",  
     convMsg("display node configuration"),
     "NONE",  "",           
     "CALL", "MAIN::do_display" ] ,

   [ "synccheck",   
     convMsg("check system file synchronization") ,
     "NONE",  "",
     "CALL", "SYNC::synccheck"    ] ,

   [ "syncgen",  
     convMsg("generate system file synchronization") ,
     "NONE",  "",
     "CALL", "SYNC::syncgen" ] ,

   [ "confshare",  
     convMsg("configure the shared environment for patching"),
     "VALUE",  convMsg("root directory of shared packages") ,
     "CALL", "OS::configure_shared" ] ,

   [ "copy",  
     convMsg("copy files from master to vice-master") ,
     "CONCATENATE",  convMsg("file list"),
     "CALL", "COPY::copy" ] ,

  ) ;
			       
#--------------------------------------------------------------
#
#  usage
#
#      print the help
#
#--------------------------------------------------------------

sub usage
  {
    print "$E_productname $E_toolname\n" ;
    printLine("Usage: nhadm [ arguments ] command") ;

    foreach my $cmd (@G_command)
      {
	my $list = "" ;
	if ($$cmd[$G_cmdpostcmd] eq "VALUE_LIST")
	  {
	    my $ref = @$cmd[$G_cmdpostpar];
	    $list = "[" ;
	    foreach my $value (@$ref)
	      {
		$list .= " " . $value ;
	      }
	      $list .= " ]" ;
	  }
	elsif ($$cmd[$G_cmdpostcmd] eq "VALUE")
	  {
	    $list = "[ <$$cmd[$G_cmdpostpar]> ]" ;
	  }
	elsif ($$cmd[$G_cmdpostcmd] eq "CONCATENATE")
	  {
	    $list = "[ <$$cmd[$G_cmdpostpar]> ]" ;
	  }
	printf "    %s %s :\n", $$cmd[$G_cmdname], $list ;
	printf  "       %s\n" , $$cmd[$G_cmdhlp];
      }

    print "\n" ;
    printLine("  options:") ;

    foreach my $tabparam (@G_all_parameters)
      {
	printf "\n    $$tabparam[$G_allhlp]:\n";
	my $paramref = @$tabparam[$G_alllst];
	foreach my $param (@$paramref)
	  {
	    printf "      %2s | %-13.13s %s\n",
	    $$param[$G_prmshort], $$param[$G_prmlong],$$param[$G_prmhlp] ;
	    if ($$param[$G_prmaction] eq "READ")
	      {
		my $name = $$param[$G_prmspecial2] ;
		if ($name ne "") {
		  printLine("                         (default: %s)", $$name);
		}
	      }
	  }
      }
  }

#--------------------------------------------------------------
#
# help
#
#      print the help
#
#--------------------------------------------------------------

sub help
  {
    usage ;
    exit 0 ;
  }

#--------------------------------------------------------------
#
#  print_header
#
#      print starting information
#
#--------------------------------------------------------------

sub print_header
  {
    LOG::printMsg(0,"%s %s", $E_productname, $E_toolname) ; ;
    chomp(my $L_date = qx/date/) ;
    LOG::printMsg(0, "Started on %s", $L_date);
    LOG::printMsg(0, "Node name: %s", $E_node_name) ;
  }

#--------------------------------------------------------------
#
#  print_footer
#
#      print test result summary
#
#--------------------------------------------------------------

sub print_footer
  {
    LOG::printMsg(0, "") ;
    chomp(my $L_date = qx/date/) ;
    LOG::printSummary ;
    LOG::printMsg(0,"Ended on: %s", $L_date) ;
  }

#--------------------------------------------------------------
#
#  get_username
#
#      get the username and set $E_isRoot
#
#--------------------------------------------------------------

sub get_username
  {
    my $result = qx{/usr/bin/id} ;
    $result =~ m/uid=[0-9]*\(([0-9a-z]*)\)/ ;
    my $username=$1 ;
    if ($username eq "root")
      {
	$E_isRoot = "TRUE" ;
      }
  }

#--------------------------------------------------------------
#
#  isInArray
#
#      return the rank of a value in an array
#
#--------------------------------------------------------------

sub isInArray
  {
    my ($array,$count,$value) = @_ ;

    my $i = 0 ;
    while ($i < $count)
      {
	if (@$array[$i]  eq $value)
	  {
	    return "TRUE" ;
	  }
      $i++ ;
      }

    return "FALSE" ;
  }

#--------------------------------------------------------------
#
#  check_one_stage
#
#      . call all functions related to a checking stage
#
#      1  name of the stage
#
#--------------------------------------------------------------

sub check_one_stage
  {
    my ($L_stage) = @_ ;

    my $arrayname = $G_check{$L_stage} ;
    foreach my $module (@$arrayname)
      {
	# module names are uppercase
	$module =~ tr/a-z/A-Z/ ;
	my $proc = "${module}::check_${L_stage}" ;
	&$proc ;
      }
  }

#--------------------------------------------------------------
#
#  do_check
#
#      . analyze the arguments of the check command
#      . execute the check command
#
#--------------------------------------------------------------

sub do_check
  {
    # argument may be missing: initialize it to avoid warning when using
    my $stage = "" ;
    ($stage) = @_ ;

    #
    #  check that it's a valid category
    #
    if ($stage ne "")
      {
	if (isInArray(\@G_stages,scalar(@G_stages),$stage) eq "FALSE")
	  {
	    printLine("Unknown stage: %s", $stage) ;
	    usage ;
	    return ;
	  }
      }


    #
    #  get the node name
    #
    my $nodename = "" ;
    if ( -e "/etc/nodename")
      {
	open(FH, "/etc/nodename") ;
	chomp($E_node_name = <FH>) ;
	close(FH) ;
      }
    else
      {
	chomp($E_node_name = qx{/usr/bin/uname -n}) ;
      }

    print_header ;
    
    #
    #  check all stages or only the requested one
    #
    my @L_stages;
    if ($stage eq "")
      {
	@L_stages = @G_stages ;
      }
    else
      {
	@L_stages = ( $stage ) ;
      }
    LOG::printMsg(0,"") ;
    my $conf_load = "FALSE";
    foreach $stage (@L_stages)
      {
	LOG::printMsg(0, "-----------------------------------------") ;
	LOG::printMsg(0,"STAGE: %s", $stage) ;
	LOG::printMsg(0,"-----------------------------------------") ;
	if (($stage eq "configuration") or ($stage eq "starting"))
	  {
	    
	    if ($conf_load eq "FALSE")
	      {
		LOG::printMsg(0,"") ;
		$conf_load = "TRUE" ;
		CONF::load_fs_conf;
		CMM::load_all_files;
	      }
	  }

	check_one_stage($stage) ;
	LOG::printMsg(0,"") ;
      }
    
    print_footer ;
  }

#--------------------------------------------------------------
#
#  do_display
#
#--------------------------------------------------------------

sub do_display
  {
    #
    #  get the node name
    #
    my $nodename = "" ;
    if ( -e "/etc/nodename")
      {
	open(FH, "/etc/nodename") ;
	chomp($E_node_name = <FH>) ;
	close(FH) ;
      }
    else
      {
	chomp($E_node_name = qx{/usr/bin/uname -n}) ;
      }

    #
    #  load all prerequesite files
    #
    CONF::load_fs_conf;
    CMM::load_all_files ;

    #
    #  do all display calls
    #

    foreach my $step (@G_display_all)
      {
	# module names are uppercase
	$step =~ tr/a-z/A-Z/ ;
	$step->display ;
      }

    return "TRUE" ;
  }

#==============================================================
#
# COMMAND PARSING
#
#==============================================================

#--------------------------------------------------------------
#
# process_command
#
#--------------------------------------------------------------

sub process_command
  {
    my ($arg) = @_ ;

    my $found = "FALSE" ;

    foreach my $cmd (@G_command)
      {
	if ($$cmd[$G_cmdname] eq $arg)
	  {
	    $G_readcmd = $cmd ;
	    $found = "TRUE" ;
	    last ;
	  }
      }

    if ($found eq "FALSE" )
      {
	printLine("%s: invalid command", $arg) ;
	exit 1;
      }

    my $type= $$G_readcmd[$G_cmdpostcmd] ;
    if (($type eq "VALUE_LIST") || ($type eq "VALUE")) {
      # force an empty string in case of no value given
      $G_postcommand[0] = "" ; 
    }
  }

#--------------------------------------------------------------
#
# process_parameter
#
#--------------------------------------------------------------

sub process_parameter
  {
    my ($param_array, $arg) = @_ ;
    my $found = "FALSE" ;

    foreach my $param (@$param_array)
      {
	if (("$$param[$G_prmshort]" eq "$arg") || ("$$param[$G_prmlong]" eq "$arg"))
	  {
	    $found = "TRUE" ;
	    if ($$param[$G_prmaction] eq "READ")
	      {
		# the following argument must read as the value
		if ($#ARGV < 0)
		  {
		    printLine("%s: value required", $arg) ;
		    exit 1;
		  }
		my $name = $$param[$G_prmspecial];
		$$name = shift @ARGV ;
	      }
	    elsif ($$param[$G_prmaction] eq "GET")
	      {
		# the following argument must read as the value: both arg and value are stored
		if ($#ARGV < 0)
		  {
		    printLine("%s: value required", $arg) ;
		    exit 1;
		  }
		my $value = shift @ARGV;
		push @G_postcommand, $arg ;
		push @G_postcommand, $value ;
	      }
	    elsif ($$param[$G_prmaction] eq "CALL")
	      {
		# when this argument is read, the function is called
		my $action = $$param[$G_prmspecial];
		&$action;
	      }
	    elsif ($$param[$G_prmaction] eq "COPY")
	      {
		# the argument is copied as is in the post command array
		push @G_postcommand, $arg ;
	      }
	    else
	      {
		print "INTERNAL ERROR, $$param[$G_prmaction]: parameter action unkown\n" ;
	      }
	    last ;
	  }
      }

    return $found ;
  }

#--------------------------------------------------------------
#
# process_precommand
#
#--------------------------------------------------------------

sub process_precommand
  {
    my ($arg) = @_ ;
    my $found = "FALSE" ;

    # search in each array listed in the global array
    foreach my $tabparam (@G_all_parameters)
      {
	my $ref = @$tabparam[$G_alllst] ;
	$found = process_parameter($ref, $arg);
	if ($found eq "TRUE")
	  {
	    last ;
	  }
      }

    if ($found eq "FALSE")
      {
	printLine("%s: unknown argument", $arg) ;
	exit 1;
      }
  }

#--------------------------------------------------------------
#
# process_postcommand
#
#--------------------------------------------------------------

sub process_postcommand
  {
    my ($arg) = @_ ;

    my $found = "FALSE" ;
    my $type = $$G_readcmd[$G_cmdpostcmd] ;

    if ($type eq "CONCATENATE")
      {
	# arguments are stored as is
	#add this one to the previously stored
	# stored in the first elem of the post-command
	$G_postcommand[0] .= $arg . " ";
      }
    elsif ($type eq "NONE")
      {
	# no argument authorized
	printLine("%s: no post-command authorized", $arg) ;
	exit 1 ;
      }
    elsif ($type eq "VALUE")
      {
	$G_postcommand[0]= $arg ;
      }
    elsif ($type eq "VALUE_LIST")
      {
	# value must be part of the list and passed as an function argument
	my $ref = $$G_readcmd[$G_cmdpostpar] ;
	foreach my $value (@$ref)
	  {
	    if ($value eq "$arg")
	      {
		$found = "TRUE" ;
		last ;
	      }
	  }
	if ($found eq "FALSE")
	  {
	    printLine("%s: unknown argument", $arg) ;
	    exit 1;
	  }
	$G_postcommand[0]= $arg ;
      }
    elsif ($type eq "PARAMETER")
      {
	# processing of a parameter list
	my $ref=@$G_readcmd[$G_cmdpostpar];
	$found = process_parameter($ref, $arg) ;
	if ($found eq "FALSE")
	  {
	    printLine("%s: unknown post-command", $arg) ;
	    exit 1;
	  }
      }
    else
      {
	print "INTERNAL ERROR, $type: command action unkown\n" ;
      }
  }

#--------------------------------------------------------------
#
#  load_command
#
#--------------------------------------------------------------

sub load_command
  {
    my $found = "FALSE" ;
    my $found_command = "FALSE" ;
    my $arg ;
    my $c ;

    while ($#ARGV >= 0)
      {
	$arg = shift @ARGV ;

	# look for the command name i.e (first arg not preceding with "-")
	my $c = substr($arg, 0, 1) ;

	if ($found_command eq "FALSE")
	  {
	    # pre-command processing
	    if (($found eq "FALSE") && ($c ne "-"))
	      {
		# not an argument: it's a command
		$found_command = "TRUE" ;
		process_command($arg) ;
	      }
	    else
	      {
		# pre-command argument: all arguments are authorized
		process_precommand($arg) ;
	      }
	  }
	else
	  {
	    # pre-command arguments: authorized arguments depend on the command
	    process_postcommand($arg) ;
	  }
      }

    if ($found_command eq "FALSE")
      {
	printLine("No command found") ;
	usage();
	exit 1 ;
      }

    #
    # perform the action
    #
    my $action = $$G_readcmd[$G_cmdspecial] ;
    if ($$G_readcmd[$G_cmdaction] eq "CALL")
      {
	# Perl function
	&$action(@G_postcommand);
      }
    else
      {
	# Shell script: building of the argument list
	my $line = "" ;
	foreach my $param (@G_postcommand)
	  {
	    $line .= " " . $param ;
	  }
	print("COMMAND=$action $line\n") ;
	system("$action $line") ;
      }
  }

#--------------------------------------------------------------
#
#  main
#
#--------------------------------------------------------------

#
#  get the username for checking if we are root
#

LOG::init;
OS::set_os_env;

get_username ;
if ($E_isRoot ne "TRUE")
  {
    printLine("Must be root") ;
  }
else
  {
    load_command ;
  }

exit $E_exit_status;
